@@ -1,12 +1,11 @@
 use clap::IntoApp;
-use clap::{App, AppSettings, Parser, Subcommand};
+use clap::{AppSettings, Parser, Subcommand};
 
 #[derive(Parser, Debug)]
 #[clap(global_setting(AppSettings::PropagateVersion))]
 #[clap(global_setting(AppSettings::DeriveDisplayOrder))]
 #[clap(global_setting(AppSettings::InferSubcommands))]
 #[clap(global_setting(AppSettings::HelpExpected))]
-#[clap(global_setting(AppSettings::UseLongFormatForHelpSubcommand))]
 #[clap(setting(AppSettings::SubcommandRequiredElseHelp))]
 #[clap(author, version, about)]
 pub struct Cli {
@@ -35,6 +34,20 @@ pub enum Commands {
     /// Get percent identity stats from a sam/bam/cram or PAF.
     ///
     /// Requires =/X operations in the CIGAR string!
+    ///
+    /// # output column descriptions:
+    ///
+    /// perID_by_matches is calculated as:
+    ///
+    ///  matches / (matches + mismatches)
+    ///
+    /// perID_by_events is calculated as:
+    ///
+    ///  matches / (matches + mismatches + insertion events + deletion events)
+    ///
+    /// perID_by_all is calculated as:
+    ///
+    ///  matches / (matches + mismatches + insertion bases + deletion bases)
     Stats {
         /// sam/bam/cram/file
         #[clap(default_value = "-")]
@@ -85,6 +98,7 @@ pub enum Commands {
     ///
     /// This is a function for lifting over coordinates from a reference (<BED>) to a query using a PAF file from minimap2 or unimap (note, you can use `paftools.js sam2paf` to convert SAM data to PAF format).
     /// The returned file is a PAF file that is trimmed to the regions in the bed file. Even the cigar in the returned PAF file is trimmed so it can be used downstream! Additionally, a tag with the format `id:Z:<>` is added to the PAF where `<>` is either the 4th column of the input bed file or if not present `chr_start_end`.
+    #[clap(aliases = &["william-t-harvey", "wth"])]
     Liftover {
         /// PAF file from minimap2 or unimap run with -c and --eqx [i.e. the PAF file must have the cg tag and use extended CIGAR opts (=/X)].
         #[clap(default_value = "-")]
@@ -118,6 +132,9 @@ pub enum Commands {
         /// Value subtracted for an insertion or deletion.
         #[clap(short, long, default_value_t = 1)]
         indel_score: i32,
+        /// Remove contained alignments as well as overlaps.
+        #[clap(short, long)]
+        remove_contained: bool,
     },
     /// Orient paf records so that most of the bases are in the forward direction.
     ///
@@ -148,9 +165,12 @@ pub enum Commands {
     /// Convert a PAF file into a SAM file. Warning, all alignments will be marked as primary!
     #[clap(visible_aliases = &["paftosam", "p2s", "paf2sam"])]
     PafToSam {
-        /// PAF file from minimap2 or unimap.
+        /// PAF file from minimap2 or unimap. Must have a CIGAR tag.
         #[clap(default_value = "-")]
         paf: String,
+        /// Optional query fasta file (with index) to populate the query seq field.
+        #[clap(short, long)]
+        fasta: Option<String>,
     },
     /// Splits fastx from stdin into multiple files.
     ///
@@ -222,6 +242,6 @@ pub fn make_cli_parse() -> Cli {
     Cli::parse()
 }
 
-pub fn make_cli_app() -> App<'static> {
-    Cli::into_app()
+pub fn make_cli_app() -> clap::Command<'static> {
+    Cli::command()
 }
