@@ -26,20 +26,18 @@ pub fn trim_paf_rec_to_rgn(rgn: &bed::Region, paf: &PafRecord) -> Option<PafReco
 
     // index at the start of trimmed alignment
     trimmed_paf.t_st = cmp::max(rgn.st, paf.t_st);
-    let start_idx = match paf.tpos_to_idx(trimmed_paf.t_st, true) {
+    let start_idx = match paf.tpos_to_idx_match(trimmed_paf.t_st, true) {
         Ok(idx) => idx,
         Err(_) => panic!(
             "\nProblem getting index in cigar:\n{}\n{}\n{}\n",
             trimmed_paf.t_st, rgn, paf
         ),
     };
-    trimmed_paf.t_st = paf.tpos_aln[start_idx];
-    trimmed_paf.q_st = paf.qpos_aln[start_idx];
 
     // index at the end of trimmed alignment
     trimmed_paf.t_en = cmp::min(rgn.en, paf.t_en);
     // the end index is not inclusive so minus 1
-    let end_idx = match paf.tpos_to_idx(trimmed_paf.t_en - 1, false) {
+    let end_idx = match paf.tpos_to_idx_match(trimmed_paf.t_en - 1, false) {
         Ok(idx) => idx,
         Err(error_idx) => panic!(
             "\nProblem getting index in cigar:\n{}\n{:?}\n{}\n{}\n",
@@ -49,18 +47,32 @@ pub fn trim_paf_rec_to_rgn(rgn: &bed::Region, paf: &PafRecord) -> Option<PafReco
             paf
         ),
     };
-    trimmed_paf.t_en = paf.tpos_aln[end_idx];
-    trimmed_paf.q_en = paf.qpos_aln[end_idx];
 
-    // if this is the case then the whole internal region is indel
-    // TODO check to make sure this is true
+    // if this is the case then the whole internal regions is an indel
     if start_idx > end_idx {
         return None;
     }
 
+    // update the start and end positions
+    trimmed_paf.t_st = paf.tpos_aln[start_idx];
+    trimmed_paf.q_st = paf.qpos_aln[start_idx];
+    trimmed_paf.t_en = paf.tpos_aln[end_idx];
+    trimmed_paf.q_en = paf.qpos_aln[end_idx];
+
     // get the cigar opts
     trimmed_paf.cigar = PafRecord::collapse_long_cigar(&paf.subset_cigar(start_idx, end_idx));
-    // trimmed_paf.long_cigar = paf.subset_cigar(start_idx, end_idx);
+
+    // make sure there are opts in the cigar that are not indels
+    let mut no_match_opts = true;
+    for opt in &trimmed_paf.cigar {
+        if matches!(opt, Match(_) | Equal(_) | Diff(_)) {
+            no_match_opts = false;
+            break;
+        }
+    }
+    if no_match_opts {
+        return None;
+    }
 
     if paf.strand == '-' {
         std::mem::swap(&mut trimmed_paf.q_en, &mut trimmed_paf.q_st);
@@ -112,7 +124,7 @@ pub fn trim_helper(name: &str, recs: &[PafRecord], rgns: &[bed::Region]) -> Vec<
         .iter()
         .cartesian_product(cur_rgns) // make all pairwise combs
         .par_bridge()
-        .filter(|(paf, rgn)| paf.paf_overlaps_rgn(rgn)) //filter to overlaping pairs
+        .filter(|(paf, rgn)| paf.paf_overlaps_rgn(rgn)) //filter to overlapping pairs
         .filter_map(|(paf, rgn)| trim_paf_rec_to_rgn(rgn, paf))
         .collect();
 
@@ -124,7 +136,7 @@ pub fn trim_paf_by_rgns(
     paf_recs: &[PafRecord],
     invert_query: bool,
 ) -> Vec<PafRecord> {
-    // swap qeury and ref if needed.
+    // swap query and ref if needed.
     let mut newvec = Vec::new();
     let recs: &[PafRecord] = if invert_query {
         for rec in paf_recs.iter() {
@@ -164,8 +176,8 @@ pub fn trim_paf_by_rgns(
 /// rec.aligned_pairs();
 /// for paf in liftover::break_paf_on_indels(&rec, 0){
 ///     eprintln!("{}", paf);
-///     assert!(paf.t_en - paf.t_st == 5, "Incorrect size.");   
-/// }   
+///     assert!(paf.t_en - paf.t_st == 5, "Incorrect size.");
+/// }
 /// ```
 pub fn break_paf_on_indels(paf: &PafRecord, break_length: u32) -> Vec<PafRecord> {
     let mut rtn = Vec::new();
@@ -183,7 +195,8 @@ pub fn break_paf_on_indels(paf: &PafRecord, break_length: u32) -> Vec<PafRecord>
                     ..Default::default()
                 };
                 //rtn.push(trim_paf_rec_to_rgn(&rgn, paf));
-                if let Some(x) = trim_paf_rec_to_rgn(&rgn, paf) {
+                if let Some(mut x) = trim_paf_rec_to_rgn(&rgn, paf) {
+                    x.check_integrity().unwrap();
                     rtn.push(x)
                 }
             }
@@ -228,7 +241,7 @@ mod tests {
             /// ------------||||I|D||
             ///             TGACGT-AC
             ///           01234567789 (forward)
-            ///               XXXXX           
+            ///               XXXXX
             ///             98765433210 (reverse)
             "
         );
