@@ -5,6 +5,7 @@ use log::LevelFilter;
 use rayon::prelude::*;
 use rust_htslib::bam;
 use rust_htslib::bam::Read;
+use rust_htslib::faidx;
 use rustybam::cli::Commands;
 use rustybam::fastx;
 use rustybam::paf::paf_swap_query_and_target;
@@ -214,6 +215,7 @@ pub fn parse_cli() {
             let rgns = bed::parse_bed(bed);
             // read in the file
             let paf = paf::Paf::from_file(paf);
+
             // trim the records
             let new_recs = liftover::trim_paf_by_rgns(&rgns, &paf.records, *qbed);
 
@@ -241,9 +243,10 @@ pub fn parse_cli() {
             match_score,
             diff_score,
             indel_score,
+            remove_contained,
         }) => {
             let mut paf = paf::Paf::from_file(paf);
-            paf.overlapping_paf_recs(*match_score, *diff_score, *indel_score);
+            paf.overlapping_paf_recs(*match_score, *diff_score, *indel_score, *remove_contained);
             for rec in &paf.records {
                 println!("{}", rec);
             }
@@ -302,11 +305,15 @@ pub fn parse_cli() {
         //
         // Run PafToSam
         //
-        Some(Commands::PafToSam { paf }) => {
+        Some(Commands::PafToSam { paf, fasta }) => {
+            let fasta_reader = fasta
+                .as_ref()
+                .map(|fasta| faidx::Reader::from_path(fasta).unwrap());
+
             let paf = paf::Paf::from_file(paf);
             println!("{}", paf.sam_header());
             for rec in paf.records {
-                println!("{}", rec.to_sam_string());
+                println!("{}", rec.to_sam_string(fasta_reader.as_ref()));
             }
         }
         //
