@@ -1,6 +1,8 @@
 use super::bed;
+use super::getfasta;
 use super::myio;
 use super::trim_overlap::trim_overlapping_pafs;
+use bio::alphabets::dna::revcomp;
 use core::{fmt, panic};
 use itertools::Itertools;
 use lazy_static::lazy_static;
@@ -9,6 +11,7 @@ use regex::Regex;
 use rust_htslib::bam::record::Cigar::*;
 use rust_htslib::bam::record::CigarString;
 use rust_htslib::bam::record::*;
+use rust_htslib::faidx;
 use std::collections::{HashMap, HashSet};
 use std::convert::TryFrom;
 use std::io::BufRead;
@@ -34,8 +37,14 @@ pub struct Paf {
     //pub records_by_contig: HashMap<String, Vec<&'a PafRecord>>,
 }
 
+impl Default for Paf {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Paf {
-    fn new() -> Paf {
+    pub fn new() -> Paf {
         Paf {
             records: Vec::new(),
             //records_by_contig: HashMap::new(),
@@ -58,7 +67,8 @@ impl Paf {
         for (index, line) in paf_file.lines().enumerate() {
             log::trace!("{:?}", line);
             match PafRecord::new(&line.unwrap()) {
-                Ok(rec) => {
+                Ok(mut rec) => {
+                    rec.check_integrity().unwrap();
                     paf.records.push(rec);
                 }
                 Err(_) => eprintln!("\nUnable to parse PAF record. Skipping line {}", index + 1),
@@ -198,10 +208,21 @@ impl Paf {
     }
 
     /// Identify overlapping pairs in Paf set
-    pub fn overlapping_paf_recs(&mut self, match_score: i32, diff_score: i32, indel_score: i32) {
+    pub fn overlapping_paf_recs(
+        &mut self,
+        match_score: i32,
+        diff_score: i32,
+        indel_score: i32,
+        remove_contained: bool,
+    ) {
+        // remove trailing indels in all cases
+        for rec in &mut self.records {
+            rec.remove_trailing_indels();
+        }
+
         let mut overlap_pairs = Vec::new();
         self.records.sort_by_key(|rec| rec.q_name.clone());
-
+        let mut contained_indexes = vec![false; self.records.len()];
         for i in 0..(self.records.len() - 1) {
             let rec1 = &self.records[i];
             let rgn1 = rec1.get_query_as_region();
@@ -216,11 +237,11 @@ impl Paf {
                     j += 1;
                     continue;
                 } else if overlap == (rec2.q_en - rec2.q_st) {
-                    //rec2.contained = true;
-                    log::trace!("{}\n^is contained in another alignment", rec1);
+                    contained_indexes[j] = true;
+                    log::debug!("{}\n^is contained in another alignment", rec2);
                 } else if overlap == (rec1.q_en - rec1.q_st) {
-                    //rec1.contained = true;
-                    log::trace!("{}\n^is contained in another alignment", rec1);
+                    contained_indexes[i] = true;
+                    log::debug!("{}\n^is contained in another alignment", rec1);
                 } else {
                     // put recs in left, right order
                     if rec1.q_st <= rec2.q_st {
@@ -234,18 +255,47 @@ impl Paf {
             }
         }
         overlap_pairs.sort_by_key(|rec| std::u64::MAX - rec.0);
-        if overlap_pairs.len() > 1 {
-            let mut left = self.records[overlap_pairs[0].1].clone();
-            let mut right = self.records[overlap_pairs[0].2].clone();
-            left.aligned_pairs();
-            right.aligned_pairs();
-            trim_overlapping_pafs(&mut left, &mut right, match_score, diff_score, indel_score);
-            log::trace!("{}", left);
-            log::trace!("{}", right);
-            self.records[overlap_pairs[0].1] = left;
-            self.records[overlap_pairs[0].2] = right;
+        log::debug!("{} overlapping pairs found", overlap_pairs.len());
+        let mut q_seen: HashSet<String> = HashSet::new();
+        let mut unseen = 0;
+        for (_overlap, i, j) in overlap_pairs {
+            let mut left = self.records[i].clone();
+            let mut right = self.records[j].clone();
+            let q_name = left.q_name.clone();
+            // if we have not seen the q_name before it cannot be
+            // in conflict with previous trimming steps
+            if !q_seen.contains(&q_name) {
+                left.aligned_pairs();
+                right.aligned_pairs();
+                trim_overlapping_pafs(&mut left, &mut right, match_score, diff_score, indel_score);
+                log::trace!("{}", left);
+                log::trace!("{}", right);
+                self.records[i] = left;
+                self.records[j] = right;
+                q_seen.insert(q_name);
+            } else {
+                unseen += 1;
+            }
+        }
+
+        if unseen > 0 {
             // recursively call for next overlap
-            self.overlapping_paf_recs(match_score, diff_score, indel_score);
+            self.overlapping_paf_recs(match_score, diff_score, indel_score, remove_contained);
+        } else if remove_contained {
+            let n_to_remove = contained_indexes.iter().filter(|&x| *x).count();
+            log::info!("Removing {} contained alignments.", n_to_remove);
+            log::info!("{} total alignments.", self.records.len());
+            let mut new_records = Vec::new();
+            assert!(self.records.len() == contained_indexes.len());
+            for (i, rec) in self.records.iter().enumerate() {
+                if !contained_indexes[i] {
+                    new_records.push(rec.clone());
+                }
+            }
+            self.records = new_records;
+            log::info!("{} total alignments.", self.records.len());
+            // remove contained records
+            //self.records.retain(|rec| !rec.contained);
         }
     }
 
@@ -430,8 +480,23 @@ impl PafRecord {
         }
     }
 
+    // make a long cigar from the existing cigar
+    pub fn make_long_cigar(&mut self) {
+        let mut long_cigar = Vec::new();
+        for opt in self.cigar.into_iter() {
+            let opt_len = opt.len() as u64;
+            for _i in 0..opt_len {
+                long_cigar.push(update_cigar_opt_len(opt, 1));
+            }
+        }
+        self.long_cigar = CigarString(long_cigar);
+    }
+
     /// This function adds matching alignment positions and cigar operations
     pub fn aligned_pairs(&mut self) {
+        // aligned pairs is messed up when there are leading or trailing indels
+        self.remove_trailing_indels();
+
         let mut t_pos = self.t_st as i64 - 1;
         let mut q_pos = self.q_st as i64 - 1;
 
@@ -447,7 +512,7 @@ impl PafRecord {
             let moves_t = consumes_reference(opt);
             let moves_q = consumes_query(opt);
             let opt_len = opt.len() as u64;
-            // incrment the ref and or query
+            // increment the ref and or query
             for _i in 0..opt_len {
                 long_cigar.push(update_cigar_opt_len(opt, 1));
                 if moves_t {
@@ -468,14 +533,22 @@ impl PafRecord {
     }
 
     /// Does a binary search on the alignment to find its index in the alignment
-    pub fn tpos_to_idx(&self, tpos: u64, right: bool) -> Result<usize, usize> {
-        let mut idx = self.tpos_aln.binary_search(&tpos)?;
-        if right {
-            while idx + 1 < self.tpos_aln.len() && self.tpos_aln[idx + 1] == tpos {
+    pub fn tpos_to_idx(&self, tpos: u64) -> Result<usize, usize> {
+        let idx = self.tpos_aln.binary_search(&tpos)?;
+        Ok(idx)
+    }
+
+    /// force the tpos index to be at a match to the right (or left)
+    pub fn tpos_to_idx_match(&self, qpos: u64, search_right: bool) -> Result<usize, usize> {
+        let mut idx = self.tpos_to_idx(qpos)?;
+        let max_idx = self.long_cigar.len();
+        // find the closes actual matching base
+        if search_right {
+            while idx < max_idx && !matches!(self.long_cigar[idx], Match(_) | Diff(_) | Equal(_)) {
                 idx += 1;
             }
         } else {
-            while idx > 0 && self.tpos_aln[idx - 1] == tpos {
+            while idx > 0 && !matches!(self.long_cigar[idx], Match(_) | Diff(_) | Equal(_)) {
                 idx -= 1;
             }
         }
@@ -491,15 +564,23 @@ impl PafRecord {
         } else {
             self.qpos_aln.binary_search(&qpos)?
         };
-        /*if right {
-            while idx + 1 < self.qpos_aln.len() && self.qpos_aln[idx + 1] == qpos {
+        Ok(idx)
+    }
+
+    /// force the qpos index to be at a match to the right (or left)
+    pub fn qpos_to_idx_match(&self, qpos: u64, search_right: bool) -> Result<usize, usize> {
+        let mut idx = self.qpos_to_idx(qpos)?;
+        let max_idx = self.long_cigar.len();
+        // find the closes actual matching base
+        if (search_right && self.strand == '+') || (!search_right && self.strand == '-') {
+            while idx < max_idx && !matches!(self.long_cigar[idx], Match(_) | Diff(_) | Equal(_)) {
                 idx += 1;
             }
         } else {
-            while idx > 0 && self.qpos_aln[idx - 1] == qpos {
+            while idx > 0 && !matches!(self.long_cigar[idx], Match(_) | Diff(_) | Equal(_)) {
                 idx -= 1;
             }
-        }*/
+        }
         Ok(idx)
     }
 
@@ -568,7 +649,9 @@ impl PafRecord {
     }
 
     pub fn remove_trailing_indels(&mut self) {
-        //let (t_bases, q_bases, nmatch, aln_len) = self.infer_n_bases();
+        // if we check integrity, here it may fail due to indels on the ends
+        // self.check_integrity().unwrap();
+
         let cigar_len = self.cigar.len();
 
         // find start to trim
@@ -587,7 +670,7 @@ impl PafRecord {
                 remove_st_q += st_opt.len();
                 // TODO learn why I need this
                 // TODO handle the case when it is a del and then and ins
-                remove_st_t += 1;
+                // remove_st_t += 1;
             }
             remove_st_opts += 1;
             removed_st_opts.push(st_opt);
@@ -597,13 +680,17 @@ impl PafRecord {
                 break;
             }
         }
+
         // remove extra counts put in my the case of Del followed by Ins
         if removed_st_opts.len() > 1 {
             for i in 0..(removed_st_opts.len() - 1) {
                 let pre_opt = removed_st_opts[i];
                 let cur_opt = removed_st_opts[i + 1];
-                if matches!(pre_opt, Del(_)) && matches!(cur_opt, Ins(_)) {
-                    remove_st_t -= 1;
+                if (matches!(pre_opt, Del(_)) && matches!(cur_opt, Ins(_)))
+                    || (matches!(pre_opt, Ins(_)) && matches!(cur_opt, Del(_)))
+                {
+                    remove_st_t += 1;
+                    remove_st_q -= 1;
                 }
             }
         }
@@ -634,10 +721,32 @@ impl PafRecord {
         if remove_en_opts > 0 || remove_st_opts > 0 {
             self.id += &format!(
                 "_TO.{}.{}",
-                CigarString(removed_st_opts),
-                CigarString(removed_en_opts)
+                CigarString(removed_st_opts.clone()),
+                CigarString(removed_en_opts.clone())
             );
         }
+
+        // some logging.
+        if remove_en_opts > 0 || remove_st_opts > 0 {
+            log::debug!(
+                "\nRemoved {} leading and {} trailing indels:\n{}\n{}\ntarget changes:{},{}\nquery changes: {},{}\n{}:{}-{}\n{}:{}-{}",
+                remove_st_opts,
+                remove_en_opts,
+                CigarString(removed_st_opts),
+                CigarString(removed_en_opts),
+                remove_st_t,
+                remove_en_t,
+                remove_st_q,
+                remove_en_q,
+                self.q_name,
+                self.q_st,
+                self.q_en,
+                self.t_name,
+                self.t_st,
+                self.t_en,
+            );
+        }
+
         // update the cigar string
         self.cigar = CigarString(self.cigar.0[remove_st_opts..].to_vec());
         self.cigar.0.truncate(self.cigar.len() - remove_en_opts);
@@ -663,6 +772,9 @@ impl PafRecord {
                 //self.remove_trailing_indels();
             }
         }
+
+        // make sure we did not break the cigar
+        self.check_integrity().unwrap();
     }
 
     pub fn truncate_record_by_query(&mut self, new_q_st: u64, new_q_en: u64) {
@@ -671,8 +783,14 @@ impl PafRecord {
         assert!(new_q_en <= self.q_en, "New end is greater than old end.");
 
         // get alignment positions
-        let mut aln_st = self.qpos_to_idx(new_q_st).unwrap();
-        let mut aln_en = self.qpos_to_idx(new_q_en - 1).unwrap();
+        self.make_long_cigar(); // needed for indel check
+        let mut aln_st = self.qpos_to_idx_match(new_q_st, true).unwrap();
+        let mut aln_en = self.qpos_to_idx_match(new_q_en - 1, false).unwrap();
+
+        let new_new_q_st = self.qpos_aln[aln_st];
+        let new_new_q_en = self.qpos_aln[aln_en] + 1; // ends are not inclusive
+
+        // if rc must swap
         if aln_st > aln_en {
             std::mem::swap(&mut aln_st, &mut aln_en);
         }
@@ -680,6 +798,7 @@ impl PafRecord {
         let new_t_en = self.tpos_aln[aln_en] + 1; // ends are not inclusive
 
         // update the cigar string
+        // update alignment positions
         self.long_cigar = self.subset_cigar(aln_st, aln_en);
         self.cigar = PafRecord::collapse_long_cigar(&self.long_cigar);
 
@@ -688,11 +807,8 @@ impl PafRecord {
         self.t_en = new_t_en;
 
         // fix the query positions that need to be
-        self.q_st = new_q_st;
-        self.q_en = new_q_en;
-
-        // update alignment positions
-        self.aligned_pairs();
+        self.q_st = new_new_q_st;
+        self.q_en = new_new_q_en;
 
         // should not happen but just in case
         self.remove_trailing_indels();
@@ -741,23 +857,60 @@ impl PafRecord {
     /// use rustybam::bed::*;
     /// use rustybam::paf::*;
     /// let paf = PafRecord::new("Q 10 0 10 + T 20 12 20 3 9 60 cg:Z:7=1X2=").unwrap();
-    /// let sam = paf.to_sam_string();
+    /// let sam = paf.to_sam_string(None);
     /// ```
-    pub fn to_sam_string(&self) -> String {
+    pub fn to_sam_string(&self, reader: Option<&faidx::Reader>) -> String {
         /*
         m64062_190807_194840/133628256/ccs	0	chr1	1	60	396=	*	0	0   *   *
         */
+        let mut clip_char = 'H';
+        let seq = match reader {
+            Some(reader) => {
+                let seq = getfasta::fetch_fasta(
+                    reader,
+                    &self.q_name,
+                    0, //self.q_st as usize,
+                    self.q_len as usize,
+                );
+                clip_char = 'S';
+                let seq = if self.strand == '-' {
+                    revcomp(seq)
+                } else {
+                    seq
+                };
+                std::str::from_utf8(&seq).unwrap().to_string()
+            }
+            None => "*".to_string(),
+        };
+        let qual = "*".to_string();
         let flag = if self.strand == '-' { 16 } else { 0 };
+        let mut leading_clip = if self.q_st > 0 {
+            format!("{}{}", self.q_st, clip_char)
+        } else {
+            "".to_string()
+        };
+        let mut trailing_clip = if self.q_len - self.q_en > 0 {
+            format!("{}{}", self.q_len - self.q_en, clip_char)
+        } else {
+            "".to_string()
+        };
+        if self.strand == '-' {
+            std::mem::swap(&mut leading_clip, &mut trailing_clip);
+        }
+        let o_cigar = format!("{}{}{}", leading_clip, self.cigar, trailing_clip);
         format!(
-            "{}\t{}\t{}\t{}\t{}\t{}H{}{}H\t*\t0\t0\t*\t*",
+            "{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}",
             self.q_name,
             flag,
             self.t_name,
             self.t_st + 1,
             self.mapq,
-            self.q_st,
-            self.cigar,
-            self.q_len - self.q_en,
+            o_cigar,
+            "*",
+            0,
+            0,
+            seq,
+            qual
         )
     }
 }
